@@ -1,6 +1,7 @@
 #include "imv.h"
 
 #include <assert.h>
+#include <fcntl.h>
 #include <ctype.h>
 #include <errno.h>
 #include <getopt.h>
@@ -537,10 +538,6 @@ struct imv *imv_create(void)
   imv->commands = imv_commands_create();
   imv->console = imv_console_create();
   imv_console_set_command_callback(imv->console, &command_callback, imv);
-  imv->ipc = imv_ipc_create();
-  if (imv->ipc) {
-    imv_ipc_set_command_callback(imv->ipc, &command_callback, imv);
-  }
   imv->title_text = strdup(
       "imv - [${imv_current_index}/${imv_file_count}]"
       " [${imv_width}x${imv_height}] [${imv_scale}%]"
@@ -754,6 +751,39 @@ static bool parse_initial_pan(struct imv *imv, const char *pan_params)
   return true;
 }
 
+static void set_cloexec(int fd)
+{
+  int flags = fcntl(fd, F_GETFD);
+  assert(flags != -1);
+  flags |= FD_CLOEXEC;
+  int rc = fcntl(fd, F_SETFD, flags);
+  assert(rc != -1);
+}
+
+static bool parse_initial_width(struct imv *imv, const char *width_value)
+{
+  char *endptr;
+  errno = 0;
+  int val_x = strtol(width_value, &endptr, 10);
+  if(errno != 0 || *endptr != '\0' || endptr == width_value) {
+    return false;
+  }
+  imv->initial_width = val_x;
+  return true;
+}
+
+static bool parse_initial_height(struct imv *imv, const char *height_value)
+{
+  char *endptr;
+  errno = 0;
+  int val_y = strtol(height_value, &endptr, 10);
+  if(errno != 0 || *endptr != '\0' || endptr == height_value) {
+    return false;
+  }
+  imv->initial_height = val_y;
+  return true;
+}
+
 static void *pipe_stdin(void *data)
 {
   int *fd = data;
@@ -834,7 +864,7 @@ bool imv_parse_args(struct imv *imv, int argc, char **argv)
   int o;
 
  /* TODO getopt_long */
-  while ((o = getopt(argc, argv, "frdxhvlu:s:n:b:t:c:w:")) != -1) {
+  while ((o = getopt(argc, argv, "frdxhvlu:s:n:b:t:c:w:W:H:")) != -1) {
     switch(o) {
       case 'f': imv->start_fullscreen = true;                    break;
       case 'r': imv->recursive_load = true;                      break;
@@ -874,6 +904,18 @@ bool imv_parse_args(struct imv *imv, int argc, char **argv)
           return false;
         }
         break;
+      case 'W':
+        if(!parse_initial_width(imv, optarg)) {
+          imv_log(IMV_ERROR, "Invalid initial width value. Aborting.\n");
+          return false;
+        }
+        break;
+      case 'H':
+        if(!parse_initial_height(imv, optarg)) {
+          imv_log(IMV_ERROR, "Invalid initial height value. Aborting.\n");
+          return false;
+        }
+        break;
       case 'c': list_append(imv->startup_commands, optarg); break;
       case 'w': parse_window_title(imv, optarg); break;
       case '?':
@@ -927,6 +969,11 @@ int imv_run(struct imv *imv)
   if (!setup_window(imv))
     return 1;
 
+  imv->ipc = imv_ipc_create();
+  if (imv->ipc) {
+    imv_ipc_set_command_callback(imv->ipc, &command_callback, imv);
+  }
+
   /* if loading paths from stdin, kick off a thread to do that - we'll receive
    * events back via internal events */
   int *stdin_pipe_fds = NULL;
@@ -942,6 +989,9 @@ int imv_run(struct imv *imv)
       return 1;
     }
 
+    /* if a child process spawned by imv inherits the write part of the pipe,
+     * load_paths_from_stdin() will not exit until the child dies */
+    set_cloexec(stdin_pipe_fds[1]);
     imv->stdin_pipe = fdopen(stdin_pipe_fds[0], "re");
 
     if (pthread_create(&load_paths_thread, NULL, load_paths_from_stdin, imv)
@@ -1048,6 +1098,11 @@ int imv_run(struct imv *imv)
               continue;
             }
 
+            if (!imv->stdin_image_data || !imv->stdin_image_data_len) {
+              /* Skip if image failed to load */
+              continue;
+            }
+
             result = backend->open_memory(imv->stdin_image_data,
                 imv->stdin_image_data_len, &new_source);
           } else {
@@ -2018,27 +2073,41 @@ static size_t read_from_stdin(void **buffer)
   size_t len = 0;
   ssize_t r;
   size_t step = 4096; /* Arbitrary value of 4 KiB */
-  void *p;
+  void *new_buf;
 
-  errno = 0; /* clear errno */
+  errno = 0;
+  *buffer = NULL;
 
-  for (*buffer = NULL; (*buffer = realloc((p = *buffer), len + step));
-      len += (size_t)r) {
-    if ((r = read(STDIN_FILENO, (uint8_t *)*buffer + len, step)) == -1) {
-      perror(NULL);
+  while (1) {
+    new_buf = realloc(*buffer, len + step);
+    if (new_buf) {
+      *buffer = new_buf;
+    } else {
+      /* Failed to extend buffer */
+      int save = errno;
+      free(*buffer);
+      errno = save;
+      *buffer = NULL;
+      len = 0;
       break;
-    } else if (r == 0) {
+    }
+
+    r = read(STDIN_FILENO, (uint8_t *)*buffer + len, step);
+    if (r > 0) {
+      len += (size_t)r;
+    } else {
+      if (r < 0) {
+        /* Read error */
+        int save = errno;
+        perror(NULL);
+        free(*buffer);
+        errno = save;
+        len = 0;
+      }
       break;
     }
   }
 
-  /* realloc(3) leaves old buffer allocated in case of error */
-  if (*buffer == NULL && p != NULL) {
-    int save = errno;
-    free(p);
-    errno = save;
-    len = 0;
-  }
   return len;
 }
 
