@@ -4,10 +4,13 @@
 #include <stdbool.h>
 
 struct imv;
+struct imv_backend;
 
 struct imv *imv_create(void);
 void imv_free(struct imv *imv);
 
+void imv_install_backend(struct imv *imv, const struct imv_backend *backend);
+
 bool imv_load_config(struct imv *imv);
 bool imv_parse_args(struct imv *imv, int argc, char **argv);
 
